AbclObject subclass: #AbclFibonacci
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
AbclFibonacci comment:
'Class AbclFibonacci implements fibonacci computation with two variants.

The first one is using two waitFor:andDo: constructs to wait for both subresults, as proposed in Abcl/R2 (Masuhara et al. 1993).

The second one is adding two future type transmissions, as proposed in Abcl/f (Taura et al. 1994).

Note that in the case of future type message passing, future objects are returned.
If the value is not computed yet for the receiver of the + operation, + implicitly waits for the assignment of the future (see class Future). But if the receiver is computed but the argument is not computed yet, + operation will fail (because it expects a numeric argument, not a future). Meanwhile, such a failure exchanges argument with receiver and retries to perform the operation. So eventually the result is to be completed with no failure.'!


!AbclFibonacci methodsFor: 'script'!

n2: n replyTo: r
	"Second version:
		Create recursively two new actors to handle the two recursive sub computations.
		Send two future type messages and sum up their values."

	n < 2
		ifTrue: [r reply: 1]
		ifFalse:
			[r reply:
				(self class new active n: n-1 replyTo: #future)
				+ (self class new active n: n-2 replyTo: #future)]!

n: n replyTo: r
	"First version:
		Create recursively two new actors to handle the two recursive sub computations.
		Wait for both of their replies."

	n < 2
		ifTrue: [r reply: 1]
		ifFalse:
			[self class new active n: n-1 replyTo: aself.
			self class new active n: n-2 replyTo:aself.
			self waitFor: #(reply:) andDo:
				[:v1 | self waitFor: #(reply:) andDo:
					[:v2 | r reply: v1 + v2]]]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbclFibonacci class
	instanceVariableNames: ''!


!AbclFibonacci class methodsFor: 'example'!

example
	"self example"

	"First version:
		Create recursively two new actors to handle the two recursive sub computations.
		Wait for both of their replies."

	self new active
		n: 10 replyTo: Print!

example2
	"self example2"

	"Second version:
		Create recursively two new actors to handle the two recursive sub computations.
		Send two future type messages and sum up their values."

	self new active
		n2: 10 replyTo: Print! !

ExpressAbclObject subclass: #ExpressCheck
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
ExpressCheck comment:
'Class ExpressCheck is a simple example to show how express messages may interrupt standard message computation.
Note that because method Express does a nonResume, if a normal message (normal) was interrupted, it is aborted.'!


!ExpressCheck methodsFor: 'script'!

Express: delay
	"This is an express mode message."

	Transcript show: 'start express'; cr.
	(Delay forMilliseconds: delay) wait.
	Transcript show: 'express stop'; cr!

ExpressNonResume: delay
	"This is an express mode message."

	Transcript show: 'start express'; cr.
	(Delay forMilliseconds: delay) wait.
	Transcript show: 'nonResume '; cr.
	self nonResume.
	Transcript show: 'express stop'; cr!

normal: delay

	Transcript show: 'start normal'; cr.
	(Delay forMilliseconds: delay) wait.
	Transcript show: 'normal stop'; cr!

normalAtomic: delay

	self expressAtomic:
		[Transcript show: 'atomic '.
		self normal: delay]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ExpressCheck class
	instanceVariableNames: ''!


!ExpressCheck class methodsFor: 'example'!

example
	"self example"

	| expressActiveObject |
	expressActiveObject := self new active.
	expressActiveObject normal: 100; ExpressNonResume: 100; normalAtomic: 100; normal: 100.
	(Delay forMilliseconds: 1500) wait.
	expressActiveObject Express: 40! !

AbclObject subclass: #AbclBoundedBuffer
	instanceVariableNames: 'contents maxSize '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
AbclBoundedBuffer comment:
'Class AbclBoundedBuffer implements the bounded buffer example.
If a buffer accepted a disabled request (e.g., a put: message once it is full), it will explicitly wait for an alternative request (e.g., a get request) in order to first free the buffer.

See definition of put: and getAndReplyTo: methods.
Note that as opposed to behavior/state replacement as in class EmptyBoundedBuffer (category Actalk-Ext-Replace-Examples), synchronization code appears explicitly within the method body. This is because acceptance of method is controlled after already accepting a disabled request.
Note that mixing up synchronization code with method code is not highly modular. This proves to be a problem for reusing specifications in subclasses.
See discussion of inheritance anomalies in examples of categories Actalk-Synchro-*.

Event handling methods associated to waitFor events are defined for tracing purposes.'!


!AbclBoundedBuffer methodsFor: 'initialize'!

initialize: anInteger
	maxSize := anInteger.
	contents := BoundedBufferObject new maxSize: maxSize! !

!AbclBoundedBuffer methodsFor: 'script'!

getAndReplyTo: r
	"If empty, then wait for a put: and serve directly the get request."

	self isEmpty
		ifTrue: [self waitFor: #(put:)	 andDo: [:newItem |
					r reply: newItem]]
		ifFalse:
			[r reply: contents removeFirst]!

put: item
	"If full, then first wait for and serve a get."

	self isFull
		ifTrue: [self waitFor: #(getAndReplyTo:) andDo: [:r |
					r reply: contents removeFirst]].
	contents addLast: item! !

!AbclBoundedBuffer methodsFor: 'printing'!

printOn: aStream
	"Print as its class name, then its contents."

	super printOn: aStream.
	contents printOn: aStream! !

!AbclBoundedBuffer methodsFor: 'events'!

eventWaitFor: arrayOfSelectors
	"Trace when entering a waitFor."

	Transcript show: self printString , ' waitFor ' , arrayOfSelectors printString; cr!

eventWaitForAccept: aMessage
	"Trace when accepting a message waitedFor."

	Transcript show: self printString , ' waitForAccept (' , aMessage compactPrintString , ')'; cr!

eventWaitForComplete: aMessage
	"Trace when completing a message waitedFor."

	Transcript show: self printString , ' waitForComplete (' , aMessage compactPrintString , ')'; cr! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbclBoundedBuffer class
	instanceVariableNames: ''!


!AbclBoundedBuffer class methodsFor: 'instance creation'!

new: maxSize
	^self new initialize: maxSize! !

!AbclBoundedBuffer class methodsFor: 'example'!

exampleSize: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleSize: 3 numberItems: 8 speedRatio: 4"
	"self exampleSize: 3 numberItems: 8 speedRatio: 1/4"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runNowGet: numberItems! !

AbclBoundedBuffer subclass: #AbclGgetBoundedBuffer
	instanceVariableNames: 'afterPut '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!


!AbclGgetBoundedBuffer methodsFor: 'script'!

ggetAndReplyTo: r
	"Nothing to reuse and VERY complex.
	This is because if the buffer is empty, we need to wait for a put,
	but then we cannot accept the gget. So we need a put; put; get; gget sequence
	but it is not the only one valid!!"! !

AbclBoundedBuffer subclass: #AbclGet2BoundedBuffer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!


!AbclGet2BoundedBuffer methodsFor: 'script'!

get2AndReplyTo: r
	"If less than two items, wait for one put request.
	If buffer was not empty, then reply the stored item and the new one.
	Otherwise we need to wait for one more put."

	contents size < 2
		ifTrue:
			[self waitFor: #(put:) andDo: [:newItem |
				self isEmpty
					ifTrue:
						[self waitFor: #(put:) andDo: [:newItem2 |
							r reply: (Array new
										with: newItem
										with: newItem2)]]
					ifFalse:
						[r reply: (Array new
									with: contents removeFirst
									with: newItem)]]]
		ifFalse:
			[r reply: contents removeFirst]! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbclGet2BoundedBuffer class
	instanceVariableNames: ''!


!AbclGet2BoundedBuffer class methodsFor: 'example'!

exampleGet2Size: maxSize numberItems: numberItems speedRatio: ratio
	"self exampleGet2Size: 3 numberItems: 8 speedRatio: 2"

	| buffer producer consumer |
	buffer := (self new: maxSize) active.
	producer := (Producer new buffer: buffer delay: 1) active.
	consumer := (Consumer new buffer: buffer delay: ratio) active.
	producer runPut: numberItems.
	consumer runNowGet2: numberItems! !

AbclObject subclass: #TreeComparator
	instanceVariableNames: 'extractor1 extractor2 input1 input2 '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
TreeComparator comment:
'TreeComparator implements pattern matching of the fringes of two trees (implemented as two Smalltalk arrays).
(This is the classical same-fringe example). This Abcl/1 version was originally described in OOPSLA''86 paper on Abcl/1. It was some derivation of algorithm introduced by Bernard Serpette in his thesis (1984) on concurrent Lisp (named Pive) with Unix-like pipes were used.

There are three actors in the algorithm.
Two extractors (TreeExtractor) extract the fringe of each tree, that is generates successive leaves.
The comparator (TreeComparator) compares the two fringes leaf by leaf.
The comparator send the initial requests to both extractors through two MAfuture type messages.
Each extractor will use its MAfuture reply destination to return each new leaf to the comparator.
In this example MAfutures are used as communication channels (as for Unix pipes).

Note that both extraction (by TreeExtractor) and comparison (by TreeComparator) methods are recursive. TreeExtractor method extract: needs to be defined as a private routine because it involves recursion and synchronization. Therefore it cannot be defined in a straightforward way as a script method. (Remember that recursion and synchronous message sending leads to deadlock, thus we would have to find alternative non trivial way of writing down the extraction process).

In case of failure (matching fails) the comparator send messages in the express mode to both extractors in order to stop them (and abort their current tree extraction by using the nonResume construct).'!


!TreeComparator methodsFor: 'initialize'!

initialize
	"Set the two tree extractors."

	extractor1 := (TreeExtractor new number: 1) active.
	extractor2 := (TreeExtractor new number: 2) active! !

!TreeComparator methodsFor: 'script'!

sameFringe: tree1 with: tree2
	"Compute a same fringe.
	Initiate the two communication channels through the two MAfuture type (MA for multiple answers) messages sent.
	Then start peeking into the two MAFuture objects used as two communication pipes."

	input1 := extractor1 fringe: tree1 replyTo: #MAfuture.
	input2 := extractor2 fringe: tree2 replyTo: #MAfuture.
	self compareNextElements! !

!TreeComparator methodsFor: 'private routines'!

compareNextElements
	"Compare elements one by one taken from the two MAFutures."

	| element1 |
	element1 := input1 next.
	element1 = (input2 next)
		ifTrue: [element1 = #EOT			"EOT = end of tree special symbol."
					ifTrue: [self success]
					ifFalse: [self compareNextElements]]
		ifFalse: [self failure]!

failure

	Print reply: false.
	extractor1 ExpressStop.		"Stop and abort the two extractors with an express message."
	extractor2 ExpressStop!

success

	Print reply: true! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TreeComparator class
	instanceVariableNames: ''!


!TreeComparator class methodsFor: 'instance creation'!

new
	^super new initialize! !

!TreeComparator class methodsFor: 'example'!

counterExample
	"TreeComparator counterExample"

	TreeComparator new active sameFringe:	#(1 (2 ((3) 40) 5) (6) (((7) 8) 9) ((10)))
									with:	#(1 2 3 4 5 6 7 8 9 10)!

example
	"TreeComparator example"

	TreeComparator new active sameFringe:	#(1 (2 ((3) 4) 5) (6) (((7) 8) 9) ((10)))
									with:	#(1 2 3 4 5 6 7 8 9 10)! !

ExpressAbclObject subclass: #TreeExtractor
	instanceVariableNames: 'output number '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!
TreeExtractor comment:
'See comment of class TreeComparator.'!


!TreeExtractor methodsFor: 'initialize'!

number: anInteger

	number := anInteger! !

!TreeExtractor methodsFor: 'script'!

ExpressStop
	"This is an express mode message."
	"Interrupt and abort current computation (message fringe:replyTo:)."

	Transcript show: self printString , ' is stopped.'; cr.
	self nonResume!

fringe: tree replyTo: aReplyDestination
	"Initialize and start extraction of elements from the tree."

	output := aReplyDestination.	"Store the reply destination, a MAFuture,"
								"used as a (permanent) communication channel."
	Transcript show: self printString , ' extract: ' , tree printString; cr.
	self extract: tree.
	output reply: #EOT	"EOT = end of tree special symbol."		"Sent after completion."! !

!TreeExtractor methodsFor: 'private routines'!

extract: tree

	Processor yield.		"To simulate simultaneity of computing between the two extractors.
						Note that event handling methods are not effective because this is a private routine."
	"Below is some standard traversal of a tree, except that it is represented as an array.
	See protocol tree simulation in class Array."
	(tree isKindOf: Array)
		ifTrue:		"A non leaf, that is a tree."
			[tree isEmpty	
				ifFalse: [self extract: tree left;
							extract: tree right]]
		ifFalse:		"A leaf."
			[Transcript show: self printString , ' : ' , tree printString; cr.
			output reply: tree]		"Return it to the comparator."! !

!TreeExtractor methodsFor: 'printing'!

printOn: aStream

	aStream nextPutAll: self class name , number printString! !

ActorCounter subclass: #AbclReplaceCounter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Actalk-Ext-Abcl-Examples'!


!AbclReplaceCounter methodsFor: 'default classes'!

addressClass
	"Creating an hybrid kind of active object
		along actor computation model
		and with Abcl/1 3 types of message send."

	^AbclAddress! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbclReplaceCounter class
	instanceVariableNames: ''!


!AbclReplaceCounter class methodsFor: 'example'!

example
	"self example inspect"

	^(self new contents: 100) active
						incr;
						incr;
						consultAndReplyTo: #now! !
